<?php

namespace Drupal\video_embed_twentythree\Plugin\video_embed_field\Provider;

use GuzzleHttp\Exception\RequestException;
use Drupal\video_embed_field\ProviderPluginBase;

/**
 * A TwentyThree provider plugin.
 *
 * @VideoEmbedProvider(
 *   id = "twentythree",
 *   title = @Translation("TwentyThree")
 * )
 */
class TwentyThree extends ProviderPluginBase {

  /**
   * {@inheritdoc}
   */
  public function renderEmbedCode($width, $height, $autoplay) {
    $domain = parse_url($this->getInput(), PHP_URL_HOST);

    $embed_code = [
      '#type' => 'video_embed_iframe',
      '#provider' => 'twentythree',
      '#url' => 'https://' . $domain . '/v.ihtml/player.html',
      '#query' => [
        'photo_id' => $this->getVideoId(),
        'autoPlay' => $autoplay
      ],
      '#attributes' => [
        'width' => $width,
        'height' => $height,
        'frameborder' => '0',
        'allowfullscreen' => 'allowfullscreen'
      ],
    ];

    if ($start = $this->getStartTime()) {
      $embed_code['#query']['start'] = $start;
    }

    return $embed_code;
  }

  /**
   * Get the TwentyThree oembed data.
   *
   * @return array|FALSE
   *   An array of data from the oembed endpoint.
   */
  protected function oEmbedData() {
    $url = $this->getInput();
    $url_parts = parse_url($url);
    $query = [
      'url' => $url,
      'format' => 'json',
    ];
    $oembed_url = $url_parts['scheme'] . '://' . $url_parts['host'] . '/oembed?';

    try {
      $request = \Drupal::httpClient()->get($oembed_url, ['query' => $query]);
      if ($request->getStatusCode() == 200) {
        return json_decode($request->getBody()->getContents());
      }
    }
    catch (RequestException $e) {
      return FALSE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->oEmbedData()->title;
  }

  /**
   * {@inheritdoc}
   */
  public function getRemoteThumbnailUrl() {
    return $this->oEmbedData()->thumbnail_url;
  }

  /**
   * {@inheritdoc}
   */
  public static function getIdFromInput($input) {
    $config = \Drupal::config('video_embed_twentythree.settings');
    $domains = str_replace("\n", '|', $config->get('video_domains'));
    $url = urldecode($input);

    $matches1 = $matches2 = [];
    preg_match("/^https?:\/\/({$domains})\/video\/(?<id>[a-z0-9]+).*$/", $url, $matches1);
    preg_match("/^https?:\/\/({$domains})\/v.ihtml\/player.html\?.*photo_id=(?<id>[0-9]+).*$/", $url, $matches2);

    if ($matches1 && !empty($matches1['id'])) {
      return (int) $matches1['id'];
    }
    else if ($matches2 && !empty($matches2['id'])) {
      return (int) $matches2['id'];
    }
    else {
      return FALSE;
    }
  }

  /**
   * Get the start time from the URL.
   *
   * @return string|FALSE
   *   A start parameter to pass to the frame or FALSE if none is found.
   */
  protected function getStartTime() {
    preg_match('/\?.*start=(?<start>(\d+)s)$/', $this->input, $matches);
    return isset($matches['start']) ? $matches['start'] : FALSE;
  }

}
